#! /usr/bin/env python2.5

from __future__ import with_statement

from boushi import Boushi
from bzv.infra import Infra
from bzv.storagecomponent import BZVStorageComponent as StorageComponent
from os import makedirs, listdir
from os.path import isdir, join
from re import compile
from sys import exit
from time import ctime, time
from urllib import urlopen

def writeBatch(sessionDirectory, number, batch):
    if not isdir(sessionDirectory):
        makedirs(sessionDirectory)

    with open(join(sessionDirectory, 'batch_%0.15d' % number), 'w') as fd:
         fd.write('\n'.join(batch))
    return True

def createBatches(storage, storageType, repository, sessionDirectory, batchSize, reindexPartName):
    currentBatch = 0
    batch = []
    identifiersFound = False
    db = StorageComponent(storage, storageType=storageType)
    for recordIdentifier in db.listIdentifiers(partName=reindexPartName, identifierPrefix=repository):
        batch.append(recordIdentifier)
        if len(batch) == batchSize:
            identifiersFound = writeBatch(sessionDirectory, currentBatch, batch)
            currentBatch += 1
            batch = []
    additionalBatch = 0
    if batch != []:
        identifiersFound = writeBatch(sessionDirectory, currentBatch, batch)
        additionalBatch = 1
    if not identifiersFound:
        return 0
    return currentBatch + additionalBatch

class Response():
    def __init__(self, url, sessionDirectory, log):
        self.response = urlopen(url)
        self.sessionDirectory = sessionDirectory
        self.log = log

    def readResponse(self):
        n = 0
        numberOfBatches = 0
        for line in self.response.readlines():
            if line.startswith("+"): 
                n += 1
                continue
            if line.startswith("!"):
                self.log.write(ctime() + line + '\n')
                print line
                self._removeLineFromBatch(n + 1)
                numberOfBatches = 1
                n = 0
                continue
            if line.startswith("=batches"): 
                line.rstrip('\n')
                numberOfBatches = int(line.split(": ")[1])
                break
        return numberOfBatches, n

    def _removeLineFromBatch(self, n):
        print "line", n, "will be removed"
        fileName = join(self.sessionDirectory, listdir(self.sessionDirectory)[0])
        newBatch = ""
        ln = 0
        batchFile = open(fileName, 'r')
        for line in batchFile.readlines():
            ln += 1
            if ln is not n:
                newBatch += line
        batchFile.close()
        batchFile = open(fileName, 'w')
        batchFile.write(newBatch)
        batchFile.close()

def main(bzvConfigFile, session='', batchSize="200", repository='', identifier=''):
    ## configuration ##
    infra = Infra('Starting Reindexer',bzvConfigFile=bzvConfigFile)
    configRoot = infra.configRoot
    config = configRoot.getSectionDict('system')
    storage = join(config['databasepath'], "storage")
    storageType = configRoot.get('storagetype', section = 'storagecomponent')
    sruUpdatePortNumber = config['sruupdateportnumber']
    reindexIdentifierBatchPath = config['reindexidentifierbatchpath']
    reindexPartName = config['reindexpartname']

    log = open(join(reindexIdentifierBatchPath, "log") , 'a')
    log.write(ctime() + " start of " + session + '\n')
    log.write(ctime() + " batchSize=" + batchSize + '\n')
    if repository: log.write(ctime() + " repository=" + repository + '\n')
    if identifier: log.write(ctime() + " identifier=" + identifier + '\n')

    sessionDirectory = join(reindexIdentifierBatchPath, session)
    numberOfBatches = 0
    if not isdir(sessionDirectory):
        numberOfBatches = 1
        if repository:
            numberOfBatches = createBatches(storage, storageType, repository, sessionDirectory, int(batchSize), reindexPartName)
            if numberOfBatches == 0: 
                log.write(ctime() + " nothing to do\n")
                log.write(ctime() + " end of " + session + '\n')
                return(0)
        elif identifier:
            writeBatch(sessionDirectory, 0, [identifier])
    else:
        log.write(ctime() + " resuming existing session\n")
        numberOfBatches = 1
    log.flush()
    url = "http://localhost:" + sruUpdatePortNumber + "/reindex?session=" + session + "&batchsize=" + batchSize
    total = 0
    t0 = time()
    while numberOfBatches > 0:
        response = Response(url, sessionDirectory, log)
        numberOfBatches, n = response.readResponse()
        total += n
        print "total", total
        print "numberOfBatches", str(numberOfBatches)
    if total > 0:
        print "total", total
        t1 = time()
        delta = t1-t0
        print "time", delta
        avg = total/delta
        print "average", avg
        log.write(ctime() + " time in seconds " + str(delta) + '\n')
        log.write(ctime() + " " + str(total) + " records indexed\n")
        log.write(ctime() + " # records per sec " + str(avg) + '\n')
    else: log.write(ctime() + " nothing to do\n")
    log.write(ctime() + " end of " + session + '\n')
    log.close()
    exit(0)

if __name__ == '__main__':
    from sys import argv
    usage = "Usage: reindexer.py /etc/bzv/bzv.cfg session=session [batchsize=batchsize] [repository=repository | identifier=identifier]"
    length = len(argv)
    if length < 3:
        print usage
        exit(1)

    bzvConfigFile = argv[1]
    args = argv[2:]
    batchSize = "200"
    repository = ''
    identifier = ''
    for arg in args:
        key, value = arg.split('=')
        if key == "session": session = value
        elif key == "batchsize": batchSize = str(value)
        elif key == "repository": repository = value 
        elif key == "identifier": identifier = value 
    if not session:
        print usage
        exit(1)
    main(bzvConfigFile, session=session, batchSize=batchSize, repository=repository, identifier=identifier)
